;;
;; SlowCD, CD Speed Control v1.04 -- main program
;;
;; Copyright (c) 2001 by Joergen Ibsen / Jibz
;; All Rights Reserved
;;

; This code was written with great help from Iczelion's win32
; assembly tutorials

; -- Version history (releases are marked with a '*') --
;
;   v1.04 *: Adjusted the code a little.
;            Switched to using CLASH library to handle options.
;   v1.03  : Drive information is shown using a dialog.
;   v1.02 *: Optimized the code a little.
;   v1.01  : Fixed some bugs. Added info when speed is omited.
;   v1.00  : Initial version. [May 28th 2001]

.586

.model flat,stdcall

option casemap:none

include windows.inc
include user32.inc
include kernel32.inc
include shell32.inc
include clash.inc
includelib user32.lib
includelib kernel32.lib
includelib shell32.lib
includelib clash.lib

include SlowCD.inc
include cdrom.inc
include wnaspi32.inc

IDI_MAINICON    equ 500
IDD_ABOUT       equ 2000
IDC_PRGINFO     equ 2001

AboutDlgProc    PROTO :HWND,:UINT,:WPARAM,:LPARAM
ScanInt         PROTO :DWORD

; -- data -----------------------------------------------------------

.data

	AppName       db "SlowCD - CD Speed Control",0

	; error and warning messages
	NotReady      db "Drive is not ready.",0
	SetErr        db "An error occured while setting the drive speed.",0
	SetWarning    db "You requested speed %lu, but the drive returns %lu.",0
	ArgErr        db "Speed argument must be between 176 and MAX_SPEED.",0

	CDInfoFormat  db "Drive information for [%lu:%lu]:",13,10,13,10,
	                 "  Drive name	: %s",13,10,
	                 "  Max speed	: %lu",13,10,
	                 "  Current speed	: %lu",0

	SyntaxText    db "You have to supply command line options:",13,10,13,10,
	                 "    SlowCD <host> <drive> [speed]",13,10,13,10,
	                 "If speed is omited, SlowCD will print info about the ",
	                 "drive. Speed should be between 176 and MAX_SPEED.",0

.data?

	hInstance     HINSTANCE ?
	CommandLine   LPSTR ?

	szBuffer1     db 256 dup (?)
	szBuffer2     db 256 dup (?)

	cdhostadapter dd ?
	cdtarget      dd ?
	cdspeed       dd ?

	curspeed      dd ?
	maxspeed      dd ?

; -- code -----------------------------------------------------------

.code

start:
	invoke GetModuleHandle, NULL
	mov hInstance,eax

	; get command line
	invoke GetCommandLine
	mov CommandLine,eax

	; parse command line (returns argc)
	invoke CL_ScanArgsX, CommandLine

	.if (eax < 3) || (eax > 4)

		; show a dialog with the syntax
		invoke DialogBoxParam, hInstance,IDD_ABOUT,0,addr AboutDlgProc,addr SyntaxText

	.else

		; get hostadapter and target from the command line
		invoke ScanInt, CL_argv[4]
		mov cdhostadapter,eax
		invoke ScanInt, CL_argv[8]
		mov cdtarget,eax

		; check if drive at host:target is ready
		invoke getDriveState, cdhostadapter,cdtarget

		.if eax != DS_READY

			invoke MessageBox, NULL,addr NotReady,addr AppName,MB_OK+MB_ICONERROR
			invoke ExitProcess, 1

		.endif

		; get current- and max-speed for drive
		invoke getCDSpeed, cdhostadapter,cdtarget
		mov curspeed,eax
		invoke getMAXCDSpeed, cdhostadapter,cdtarget
		mov maxspeed,eax

		.if CL_argc == 3

			; get drive name
			invoke getDriveName, cdhostadapter,cdtarget,addr szBuffer2

			; gather information into a single string
			mov eax,cdhostadapter
			mov edx,cdtarget
			invoke wsprintf, addr szBuffer1,addr CDInfoFormat,eax,edx,addr szBuffer2,maxspeed,curspeed

			; show a dialog with the information
			invoke DialogBoxParam, hInstance,IDD_ABOUT,0,addr AboutDlgProc,addr szBuffer1

		.else

			; get the desired speed from the command line
			invoke ScanInt, CL_argv[12]
			mov cdspeed,eax

			; if the desired speed is within reasonable limits ..
			.if (eax >= SPEED_1X) && (eax <= maxspeed)

				; attempt to set the speed
				invoke setCDSpeed, cdhostadapter,cdtarget,cdspeed

				.if eax != SS_COMP

					invoke MessageBox, NULL,addr SetErr,addr AppName,MB_OK+MB_ICONERROR

				.endif

				; get the speed so we can check it
				invoke getCDSpeed, cdhostadapter,cdtarget

				.if cdspeed != eax

					invoke wsprintf, addr szBuffer1,addr SetWarning,cdspeed,eax

					; if speeds do not match, show a messagebox warning
					invoke MessageBox, NULL,addr szBuffer1,addr AppName,MB_OK+MB_ICONWARNING

				.endif

			.else

				invoke MessageBox, NULL,addr ArgErr,addr AppName,MB_OK+MB_ICONERROR

			.endif

		.endif

	.endif

	; exit
	invoke ExitProcess, NULL

; ===================================================================

AboutDlgProc proc hDlg   :HWND,
                  wMsg   :UINT,
                  wParam :WPARAM,
                  lParam :LPARAM

	LOCAL hStatic :HWND

	.if wMsg == WM_COMMAND

		.if wParam == IDOK

			; end the dialog
			invoke EndDialog, hDlg, wParam

		.endif

	.elseif wMsg == WM_INITDIALOG

		; get handle of static for info
		invoke GetDlgItem, hDlg,IDC_PRGINFO
		mov hStatic,eax

		; set info text
		invoke SetWindowText, hStatic,lParam

	.endif

	xor eax,eax
	ret

AboutDlgProc endp

; ===================================================================

ScanInt proc uses edx esi src:DWORD

	mov esi,src
	xor eax,eax
	cdq                       ; edx = 0

@@:
	imul edx,10               ;
	add edx,eax               ; 'shift' in number in eax

	lodsb

	sub al,'0'                ;
	cmp al,9                  ; check if char in 0..9
	jbe @B                    ;

	xchg eax,edx              ; result in eax
	ret

ScanInt endp

; ===================================================================

end start
