;;
;; ASPI CD functions for setting cd speed
;;
;; This code is heavily inspired by the sources of CD-ROM Tool
;; which is Copyright (C) 1999-2000 Naiobrin Software:
;;
;;   http://www.naiobrin.f2s.com/cdromtool/
;;

.586

.model flat,stdcall

option casemap:none

include windows.inc
include gdi32.inc
include user32.inc
include kernel32.inc
include shell32.inc
includelib gdi32.lib
includelib user32.lib
includelib kernel32.lib
includelib shell32.lib

include SlowCD.inc
include cdrom.inc

include wnaspi32.inc
includelib wnaspi32.lib

ExecSCSICommand PROTO

; CD-ROM commands
ISDRIVEREADY     equ 000h
SETSPEED         equ 0BBh
INQUIRY          equ 012h
MODESENSE        equ 05Ah
MODESELECT       equ 055h

.data?

	buffer           db 256 dup (?)

	srb              SRB_ExecSCSICmd <>

.code

FillMem proc uses ecx edi dest:DWORD,len:DWORD,fill:BYTE

	mov edi,dest
	mov ecx,len
	mov al,fill
	rep stosb

	ret

FillMem endp

; -------------------------------------------------------------------

ExecSCSICommand proc

	LOCAL hEvent:HANDLE
	LOCAL counter:DWORD

	; create an event object
	invoke CreateEvent, NULL,TRUE,FALSE,NULL
	mov hEvent,eax

	MOVM srb.SRB_PostProc,hEvent

	; we will loop three times trying to communicate with the drive
	mov counter,3

command_loop:

	invoke ResetEvent, hEvent

	invoke SendASPI32Command, addr srb

	.if eax == SS_COMP

		; if completed, close the handle and return status
		invoke CloseHandle, hEvent

		movzx eax,srb.SRB_Status
		ret

	.elseif eax == SS_PENDING

		; if pending, wait for the event object
		invoke WaitForSingleObject, hEvent,INFINITE

		.if eax != WAIT_TIMEOUT

			invoke CloseHandle, hEvent

			movzx eax,srb.SRB_Status
			ret

		.endif

	.else

		invoke CloseHandle, hEvent

		mov eax,-1
		ret

	.endif

	dec counter
	jnz command_loop

	invoke CloseHandle, hEvent

	mov eax,-1
	ret

ExecSCSICommand endp

; -------------------------------------------------------------------

setupSRB proc hostadapter:DWORD,target:DWORD

	invoke FillMem, addr srb,sizeof SRB_ExecSCSICmd,0
	invoke FillMem, addr buffer,256,0FFh

	mov srb.SRB_Cmd,SC_EXEC_SCSI_CMD
	mov eax,hostadapter
	mov srb.SRB_HaId,al
	mov eax,target
	mov srb.SRB_Target,al
	mov srb.SRB_Lun,0
	mov srb.SRB_Flags,SRB_DIR_IN or SRB_EVENT_NOTIFY
	mov srb.SRB_BufLen,256
	mov srb.SRB_BufPointer,offset buffer
	mov srb.SRB_SenseLen,SENSE_LEN
	mov srb.SRB_CDBLen,12

	ret

setupSRB endp

; -------------------------------------------------------------------

getDriveState proc hostadapter:DWORD,target:DWORD

	invoke setupSRB, hostadapter,target

	mov srb.CDBByte[0],ISDRIVEREADY

	invoke ExecSCSICommand

	.if eax != SS_COMP

		mov eax,DS_NOTREADY
		ret

	.endif

	mov eax,DS_READY
	ret

getDriveState endp

; -------------------------------------------------------------------

getDriveName proc uses ecx esi edi hostadapter:DWORD,target:DWORD,dest:DWORD

	invoke setupSRB, hostadapter,target

	invoke FillMem, addr buffer,256,0

	mov srb.SRB_BufLen,100
	mov srb.SRB_CDBLen,6
	mov srb.CDBByte[0],INQUIRY
	mov srb.CDBByte[4],100

	invoke ExecSCSICommand

	.if eax != SS_COMP

		mov eax,-1
		ret

	.endif

	push eax

	; copy drive name into destination string
	lea esi,buffer[8]
	mov edi,dest
	mov ecx,28
	rep movsb

	; zero-terminate it
	xor eax,eax
	stosb

	pop eax

	ret

getDriveName endp

; -------------------------------------------------------------------

getCDSpeed proc hostadapter:DWORD,target:DWORD

	invoke setupSRB, hostadapter,target

	mov srb.CDBByte[0],MODESENSE
	mov srb.CDBByte[2],02Ah
	mov srb.CDBByte[7],001h
	mov srb.CDBByte[8],000h

	invoke ExecSCSICommand

	.if eax != SS_COMP

		mov eax,-1
		ret

	.endif

	; get the speed into eax
	xor eax,eax
	mov ah,buffer[22]
	mov al,buffer[23]
	ret

getCDSpeed endp

; -------------------------------------------------------------------

getMAXCDSpeed proc hostadapter:DWORD,target:DWORD

	invoke setupSRB, hostadapter,target

	mov srb.CDBByte[0],MODESENSE
	mov srb.CDBByte[2],02Ah
	mov srb.CDBByte[7],001h
	mov srb.CDBByte[8],000h

	invoke ExecSCSICommand

	.if eax != SS_COMP

		mov eax,-1
		ret

	.endif

	; get the max-speed into eax
	xor eax,eax
	mov ah,buffer[16]
	mov al,buffer[17]
	ret

getMAXCDSpeed endp

; -------------------------------------------------------------------

setCDSpeed proc hostadapter:DWORD,target:DWORD,speed:DWORD

	invoke setupSRB, hostadapter,target

	mov srb.SRB_Flags,SRB_DIR_OUT or SRB_EVENT_NOTIFY
	mov srb.SRB_BufLen,0
	mov srb.SRB_BufPointer,0
	mov srb.CDBByte[0],SETSPEED
	mov eax,speed
	mov srb.CDBByte[2],ah
	mov srb.CDBByte[3],al
	mov srb.CDBByte[4],0FFh
	mov srb.CDBByte[5],0FFh

	invoke ExecSCSICommand

	ret

setCDSpeed endp

end
